page: filter login/logout classes on any html element
[minimedit.git] / page.php
index 76a99b5b8aa5ede7689faa37d1e0f68b895b816e..4bea9c6d179369efd58a88ef2c32b2dd3028224c 100644 (file)
--- a/page.php
+++ b/page.php
@@ -8,16 +8,40 @@ function abort($body, $status = NULL) {
        exit;
 }
 
-function getoutput($blocks = [])
+function placeholder_include($name, $params = [])
 {
-       $rep = [];
-       foreach ($blocks as $name => $html) {
-               $rep["[[$name]]"] = sprintf('<!--BLOCK:%s-->%s<!--/-->',
-                       is_numeric($name) ? '' : "[[$name]]",
-                       preg_replace('{<!--[^-]*-->}', '', $html)
+       $path = stream_resolve_include_path("widget/$name.php");
+       if (!file_exists($path)) {
+               return '<strong class="warn"><em>'.$name.'</em> ontbreekt</strong>';
+       }
+
+       ob_start();
+       $Page = $GLOBALS['Page'] . $GLOBALS['Args'];
+       $Args = '';
+       $Place = $GLOBALS['Place'];
+       foreach ($params as $param) {
+               if ($set = strpos($param, '=')) {
+                       $Place[ substr($param, 0, $set) ] = substr($param, $set + 1);
+               }
+               elseif (!empty($param)) {
+                       $Args .= '/'.$param;
+               }
+       }
+       try {
+               include "widget/$name.php";
+               return ob_get_clean();
+       }
+       catch (Exception $e) {
+               return sprintf('<strong class="warn">%s</strong>',
+                       "fout in <em>$name</em>: {$e->getMessage()}"
                );
        }
+}
+
+function getoutput($blocks = [])
+{
        $doc = ob_get_clean();
+
        if (!empty($blocks['warn'])) {
                $warn = '<p class="warn">[[warn]]</p>';
                if ($offset = strpos($doc, '</h2>')) {
@@ -27,19 +51,46 @@ function getoutput($blocks = [])
                        $doc = $warn . "\n\n" . $doc;
                }
        }
-       return str_replace(array_keys($rep), array_values($rep), $doc);
+
+       # keep either login or logout parts depending on user level
+       global $User;
+       $hideclass = empty($User) ? 'login' : 'logout';
+       $doc = preg_replace('{\s*<([a-z]+) class="'.$hideclass.'">.*?</\1>}s', '', $doc);
+
+       return preg_replace_callback(
+               '{ (?<! <!--BLOCK: ) \[\[ ([^] ]+) ([^]]*) \]\] }x',
+               function ($sub) use ($blocks) {
+                       list ($placeholder, $name, $params) = $sub;
+                       if (isset($blocks[$name])) {
+                               $html = $blocks[$name];
+                       }
+                       else {
+                               $html = placeholder_include($name, explode(' ', $params));
+                       }
+                       return sprintf('<!--BLOCK:%s-->%s<!--/-->',
+                               is_numeric($name) ? '' : $placeholder, # edit replacement
+                               preg_replace('{<!--[^-]*-->}', '', $html) # contents
+                       );
+               },
+               $doc
+       );
 }
 
 # custom error handling
 
-define('DOCROOT', getcwd().'/');
+define('DOCROOT', getcwd());
+set_include_path(implode(PATH_SEPARATOR, [ DOCROOT, __DIR__ ]));
 
 function fail($error)
 {
        http_response_code(500);
+       if (!isset($Article)) {
+               $Article = new ArchiveArticle(NULL);
+               $Article->title = 'Fout';
+       }
        include_once 'page.inc.php';
        ob_start();
-       require_once DOCROOT.'500.html';
+       require_once '500.inc.html';
        print getoutput(['debug' => $error]);
 }
 
@@ -66,6 +117,7 @@ error_reporting(error_reporting() & ~E_FATAL);
 
 # user login and control
 
+$User = NULL;
 include_once 'auth.inc.php';
 $Edit = isset($_GET['edit']);
 
@@ -86,7 +138,7 @@ while (TRUE) {
                $PageAccess = $Page;
        }
 
-       if (file_exists("$Page.php")) {
+       if (file_exists("$Page/index.php")) {
                break;
        }
 
@@ -98,40 +150,60 @@ while (TRUE) {
        }
 }
 
-# load static contents
-
-ob_start(); # page body
-ob_start(); # inner html
-print '<div class="static">'."\n\n";
-
-$found = FALSE;
+$staticpage = NULL;
 if (file_exists("$Page$Args.html")) {
-       $found = include "./$Page$Args.html";
+       $staticpage = "$Page$Args.html";
+       if (is_link($staticpage)) {
+               $target = preg_replace('/\.html$/', '', readlink($staticpage));
+               header("HTTP/1.1 302 Shorthand");
+               header("Location: $target");
+               exit;
+       }
 }
 elseif (file_exists("$Page$Args/index.html")) {
-       $found = include "./$Page$Args/index.html";
+       $staticpage = "$Page$Args/index.html";
 }
-elseif (!empty($User['admin'])) {
-       $found = include (file_exists("$Page/template.html") ? "$Page/template.html" : './template.html');
+elseif ($User and $User->admin("edit $Page$Args")) {
+       $staticpage = (file_exists("$Page/template.inc.html") ? "$Page/template.inc.html" : 'template.inc.html');
 }
 
-print "</div>\n\n";
+# prepare page contents
 
-# execute dynamic code
+require_once('article.inc.php');
+$Article = new ArchiveArticle($staticpage);
 
-if ($Page) {
-       $found |= require "./$Page.php";
+ob_start(); # page body
+$Place = [
+       'user'  => $User ? $User->login : '',
+       'url'   => htmlspecialchars($_SERVER['REQUEST_URI']),
+];
+
+if (isset($Article->raw)) {
+       if ($User and $User->admin("edit $Page$Args")) {
+               # restore meta tags in static contents for editing
+               foreach (array_reverse($Article->meta) as $metaprop => $val) {
+                       $Article->raw = sprintf(
+                               '<meta property="%s" content="%s" />'."\n",
+                               $metaprop, $val
+                       ) . $Article->raw;
+               }
+       }
+       $Article->raw = '<div class="static">'."\n\n".$Article->raw."</div>\n\n";
 }
 
-# global html
+# output dynamic and/or static html
 
-include_once 'page.inc.php';
-
-if (!$found) {
-       # no resulting output
-       http_response_code(404);
-       ob_start();
-       @require "./404.html";
-       print getoutput([ 'url' => htmlspecialchars($_SERVER['REQUEST_URI']) ]);
+if (!$Page or require("./$Page/index.php")) {
+       # static contents
+       if (isset($Article->raw)) {
+               print $Article->raw;
+       }
+       else {
+               # no resulting output
+               http_response_code(404);
+               @require '404.inc.html';
+       }
 }
 
+include_once 'page.inc.php';
+